local addon, shared = ...

MINIMAPDOCKER = {ui = {frmw = 0}}
local DK = MINIMAPDOCKER

local _ui = MINIMAPDOCKER.ui
local docked_icons = {}
local docked_anchors = {}
local configshown = false

local DK_MODE_FLOAT = 1
local DK_MODE_BAR = 2

local DK_POS_LT = 1
local DK_POS_MM = 2
local DK_POS_RB = 3

local DK_DOCK_TP = 1
local DK_DOCK_RT = 2
local DK_DOCK_BT = 3
local DK_DOCK_LT = 4

local DK_FLOAT_U = 1
local DK_FLOAT_R = 2
local DK_FLOAT_D = 3
local DK_FLOAT_L = 4

DK._ui = _ui

local default_settings = {
	mmx=0, mmy=0, cfgx=400, cfgy=400,
	locked = false, button_size = 48, labels = true, direction = 'down', autohide=true, mode=DK_MODE_FLOAT,
	dock_edge = DK_DOCK_TP, float_dir = DK_FLOAT_D,
	dock_preferences = {}
	}

local BUTTON_SIZE = default_settings.button_size

local container_frame_l
local container_frame_r
local container_frame_c

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}			
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function DK.Test()
	for t=0, 31 do
		local f = UI.CreateFrame("Frame", "f", _ui.context)
		x = (4*t)/255
		f:SetBackgroundColor(x,x,x)
		DK.Register("Test#"..t, f)
	end
end


function DK.FixDockedLayout()
	if Docker_Settings.mode == DK_MODE_FLOAT then
		DK.LayoutFloat()
	else
		DK.LayoutMenuBar()
	end
end

function DK.ChangeLayout()
	_ui.button.MouseDown = false
	if Docker_Settings.mode == DK_MODE_FLOAT then
		Docker_Settings.mode = DK_MODE_BAR
		MINIMAPDOCKER._ui.menubar.bar:SetVisible(true)
		_ui.button:SetPoint("TOPLEFT", _ui.menubar.bar, "TOPLEFT", 0,0)
		container_frame_l = _ui.button
	else
		MINIMAPDOCKER._ui.menubar.bar:SetVisible(false)
		Docker_Settings.mode = DK_MODE_FLOAT
		_ui.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Docker_Settings.mmx, Docker_Settings.mmy)
	end	
	for k,v in pairs(docked_icons) do
		v.o:SetVisible(false)
		v.f:SetVisible(true)
	end
	_ui.cfg_docklockcb:SetChecked(false)
	DK.FixDockedLayout()
	if Docker_Settings.locked or Docker_Settings.mode == DK_MODE_BAR then
		_ui.button:SetTexture("Rift", "btn_ReturnMail_(normal).png.dds")
		_ui.frame:SetVisible(true)
	else
		_ui.button:SetTexture("Rift", "btn_ReturnMail_(disable).png.dds")
		_ui.frame:SetVisible(false)
	end
end

function DK.LayoutMenuBar()
	local frref
	local cnt_lt = 0
	local cnt_mm = 0
	local cnt_rb = 0
	
	container_frame_l = _ui.button
	container_frame_c = _ui.menubar.container_c
	container_frame_r = _ui.menubar.container_r
	
	for k,v in ipairs(docked_icons) do
		if docked_icons[k].a ~= "TOPLEFT" then
			v.f:ClearPoint(docked_icons[k].a)
		end
		if Docker_Settings.dock_preferences[v.n] == nil then
			Docker_Settings.dock_preferences[v.n] = DK_POS_LT
			frref = DK_POS_MM
		else
			frref = Docker_Settings.dock_preferences[v.n]
		end
		
		if frref == DK_POS_LT then
			cnt_lt = cnt_lt + 1
			if Docker_Settings.dock_edge == DK_DOCK_LT or Docker_Settings.dock_edge == DK_DOCK_RT then
				v.f:SetPoint("TOPLEFT", container_frame_l, "BOTTOMLEFT", 0,0)
			else
				v.f:SetPoint("TOPLEFT", container_frame_l, "TOPRIGHT",0,0)
			end
			container_frame_l = v.f
			docked_icons[k].a = "TOPLEFT"
		elseif frref == DK_POS_MM then
			cnt_mm = cnt_mm + 1
			if cnt_mm == 1 then
				v.f:SetPoint("TOPLEFT", container_frame_c, "TOPLEFT", 0,0)
			else
				if Docker_Settings.dock_edge == DK_DOCK_LT or Docker_Settings.dock_edge == DK_DOCK_RT then
					v.f:SetPoint("TOPLEFT", container_frame_c, "BOTTOMLEFT", 0,0)
				else
					v.f:SetPoint("TOPLEFT", container_frame_c, "TOPRIGHT",0,0)
				end
			end
			container_frame_c = v.f
			docked_icons[k].a = "TOPLEFT"			
		else
			cnt_rb = cnt_rb + 1
			if cnt_rb == 1 then
				v.f:SetPoint("TOPLEFT", container_frame_r, "TOPLEFT", 0,0)
			else
				if Docker_Settings.dock_edge == DK_DOCK_LT or Docker_Settings.dock_edge == DK_DOCK_RT then
					v.f:SetPoint("TOPLEFT", container_frame_r, "BOTTOMLEFT", 0,0)
				else
					v.f:SetPoint("TOPLEFT", container_frame_r, "TOPRIGHT",0,0)
				end
			end
			container_frame_r = v.f
			docked_icons[k].a = "TOPLEFT"			
		end
		v.l:SetVisible(false)
	end
	
	_ui.menubar.container_l:ClearAll()
	_ui.menubar.container_c:ClearAll()
	_ui.menubar.container_r:ClearAll()
	_ui.menubar.bar:ClearAll()
	
	if Docker_Settings.dock_edge == DK_DOCK_TP or Docker_Settings.dock_edge == DK_DOCK_BT then
		if Docker_Settings.dock_edge == DK_DOCK_TP then
			_ui.menubar.container_l:SetPoint("TOPLEFT", _ui.menubar.bar, "TOPLEFT", 0,0)
			_ui.menubar.container_c:SetPoint("TOPCENTER", _ui.menubar.bar, "TOPCENTER", 0,0)
			_ui.menubar.container_r:SetPoint("TOPRIGHT", _ui.menubar.bar, "TOPRIGHT", 0,0)
			_ui.menubar.bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0,0)
		else
			_ui.menubar.container_l:SetPoint("BOTTOMLEFT", _ui.menubar.bar, "BOTTOMLEFT", 0,0)
			_ui.menubar.container_c:SetPoint("BOTTOMCENTER", _ui.menubar.bar, "BOTTOMCENTER", 0,0)
			_ui.menubar.container_r:SetPoint("BOTTOMRIGHT", _ui.menubar.bar, "BOTTOMRIGHT", 0,0)		
			_ui.menubar.bar:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 0,0)
		end
		_ui.menubar.container_l:SetHeight(BUTTON_SIZE)
		_ui.menubar.container_l:SetWidth((cnt_lt*BUTTON_SIZE)+_ui.button:GetWidth())
		_ui.menubar.container_c:SetHeight(BUTTON_SIZE)
		_ui.menubar.container_c:SetWidth(cnt_mm*BUTTON_SIZE)
		_ui.menubar.container_r:SetHeight(BUTTON_SIZE)
		_ui.menubar.container_r:SetWidth(cnt_rb*BUTTON_SIZE)
		_ui.menubar.bar:SetHeight(BUTTON_SIZE)
		_ui.menubar.bar:SetWidth(UIParent:GetWidth())
	else
		if Docker_Settings.dock_edge == DK_DOCK_LT then
			_ui.menubar.container_l:SetPoint("TOPLEFT", _ui.menubar.bar, "TOPLEFT", 0,0)
			_ui.menubar.container_c:SetPoint("CENTERLEFT", _ui.menubar.bar, "CENTERLEFT", 0,0)
			_ui.menubar.container_r:SetPoint("BOTTOMLEFT", _ui.menubar.bar, "BOTTOMLEFT", 0,0)
			_ui.menubar.bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0,0)
		else
			_ui.menubar.container_l:SetPoint("TOPRIGHT", _ui.menubar.bar, "TOPRIGHT", 0,0)
			_ui.menubar.container_c:SetPoint("CENTERRIGHT", _ui.menubar.bar, "CENTERRIGHT", 0,0)
			_ui.menubar.container_r:SetPoint("BOTTOMRIGHT", _ui.menubar.bar, "BOTTOMRIGHT", 0,0)		
			_ui.menubar.bar:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", 0,0)
		end	
		_ui.menubar.container_l:SetHeight((cnt_lt*BUTTON_SIZE)+_ui.button:GetHeight())
		_ui.menubar.container_l:SetWidth(BUTTON_SIZE)
		_ui.menubar.container_c:SetHeight(cnt_mm*BUTTON_SIZE)
		_ui.menubar.container_c:SetWidth(BUTTON_SIZE)
		_ui.menubar.container_r:SetHeight(cnt_rb*BUTTON_SIZE)
		_ui.menubar.container_r:SetWidth(BUTTON_SIZE)
		_ui.menubar.bar:SetWidth(BUTTON_SIZE)
		_ui.menubar.bar:SetHeight(UIParent:GetHeight())
	end
end

function DK.LayoutFloat()
	local ui_x = UIParent:GetWidth()
	local ui_y = UIParent:GetHeight()
	local x_seg, y_seg
	local x_off, y_off
	local showtext = true
	local frmanchor
	local frmtarget
	local expanchor
	local exptarget
	local targtype

	local fr_anchor, fr_target
	local bg_anchor, bg_target
	
	x_off = 0
	y_off = 0
	
	local ncols = 0
	local nrows = 0
	
	if Docker_Settings.mmx > (ui_x/2) then
		x_seg = "RIGHT"
	else
		x_seg = "LEFT"
	end

	if Docker_Settings.mmy > (ui_y/2) then
		y_seg = "BOTTOM"
	else
		y_seg = "TOP"
	end

	if Docker_Settings.float_dir == DK_FLOAT_U then
		tartype = "LABEL"
		if x_seg == "LEFT" then
			frmanchor = "BOTTOMLEFT" 
			frmtarget = "TOPLEFT"
			expanchor = "BOTTOMLEFT"
			exptarget = "BOTTOMRIGHT"
			fr_anchor = "BOTTOMLEFT"
			fr_target = "BOTTOMLEFT"
			bg_anchor = "BOTTOMLEFT"
			bg_target = "TOPLEFT"
			x_off = _ui.frmw
		else
			frmanchor = "BOTTOMRIGHT"
			frmtarget = "TOPRIGHT"
			expanchor = "BOTTOMRIGHT"
			exptarget = "BOTTOMLEFT"
			fr_anchor = "BOTTOMRIGHT"
			fr_target = "BOTTOMRIGHT"
			bg_anchor = "BOTTOMRIGHT"
			bg_target = "TOPRIGHT"
			x_off = 0-_ui.frmw
		end
	elseif Docker_Settings.float_dir == DK_FLOAT_D then
		tartype = "LABEL"
		if x_seg == "LEFT" then
			frmanchor = "TOPLEFT"
			frmtarget = "BOTTOMLEFT"
			expanchor = "TOPLEFT"
			exptarget = "TOPRIGHT"
			fr_anchor = "TOPLEFT"
			fr_target = "TOPLEFT"
			bg_anchor = "TOPLEFT"
			bg_target = "BOTTOMLEFT"
			x_off = _ui.frmw
		else
			frmanchor = "TOPRIGHT"
			frmtarget = "BOTTOMRIGHT"
			expanchor = "TOPRIGHT"
			exptarget = "TOPLEFT"
			fr_anchor = "TOPRIGHT"
			fr_target = "TOPRIGHT"
			bg_anchor = "TOPRIGHT"
			bg_target = "BOTTOMRIGHT"		
			x_off = 0-_ui.frmw
		end
	elseif Docker_Settings.float_dir == DK_FLOAT_R then
		tartype = "ICON"
		showtext = false
		if y_seg == "TOP" then
			frmanchor = "TOPLEFT"
			frmtarget = "TOPRIGHT"
			fr_anchor = "TOPLEFT"
			fr_target = "TOPLEFT"
			bg_anchor = "TOPLEFT"
			bg_target = "TOPRIGHT"		
			expanchor = "TOPLEFT"
			exptarget = "BOTTOMLEFT"
		else
			frmanchor = "BOTTOMLEFT"
			frmtarget = "BOTTOMRIGHT"
			fr_anchor = "BOTTOMLEFT"
			fr_target = "BOTTOMLEFT"
			bg_anchor = "BOTTOMLEFT"
			bg_target = "BOTTOMRIGHT"		
			expanchor = "BOTTOMLEFT"
			exptarget = "TOPLEFT"
		end
	elseif Docker_Settings.float_dir == DK_FLOAT_L then
		tartype = "ICON"
		showtext = false
		if y_seg == "TOP" then
			frmanchor = "TOPRIGHT"
			frmtarget = "TOPLEFT"
			fr_anchor = "TOPRIGHT"
			fr_target = "TOPRIGHT"
			bg_anchor = "TOPRIGHT"
			bg_target = "TOPLEFT"		
			expanchor = "TOPRIGHT"
			exptarget = "BOTTOMRIGHT"
		else
			frmanchor = "BOTTOMRIGHT"
			frmtarget = "BOTTOMLEFT"
			fr_anchor = "BOTTOMRIGHT"
			fr_target = "BOTTOMRIGHT"
			bg_anchor = "BOTTOMRIGHT"
			bg_target = "BOTTOMLEFT"		
			expanchor = "BOTTOMRIGHT"
			exptarget = "TOPRIGHT"
		end
	end
		
	if Docker_Settings.labels == false then
		showtext = false
		tartype = "ICON"
		x_off = 0
		y_off = 0
	end
	
	local cnt = 0
	local rst = 1
	for k,v in ipairs(docked_icons) do
		if v.a ~= frmanchor then
			v.f:ClearPoint(v.a)
			docked_icons[k].a = frmanchor
		end
		cnt = cnt+1
		if showtext then
			v.l:SetWidth(_ui.frmw)
		end
		v.l:SetVisible(showtext)
		if k == 1 then
			v.f:SetPoint(frmanchor, _ui.button, frmtarget, 0,0)
		else
			if cnt == 17 then
				if v.a ~= expanchor then
					v.f:ClearPoint(v.a)
					docked_icons[k].a = expanchor
				end
				v.f:SetPoint(expanchor, docked_icons[k-16].f, exptarget, x_off,y_off)
				cnt = 1
				rst = rst+1
			else
				v.f:SetPoint(frmanchor, docked_icons[k-1].f, frmtarget, 0,0)
			end
		end
	end
	
	local hadj = 0
	local wadj = 0
	
	if Docker_Settings.float_dir == DK_FLOAT_U or Docker_Settings.float_dir == DK_FLOAT_D then
		ncols = rst
		hadj = 32
		if #docked_icons > 16 then
			nrows = 16
		else
			nrows = #docked_icons
		end
	else
		wadj = 32
		nrows = rst
		if #docked_icons > 16 then
			ncols = 16
		else
			ncols = #docked_icons
		end
	end
	
	local frm_h = nrows * Docker_Settings.button_size
	local frm_w = ncols * Docker_Settings.button_size
	
	_ui.frame:SetHeight(frm_h + hadj)
	if showtext then
		_ui.frame:SetWidth(wadj + frm_w + (ncols * _ui.frmw))
	else
		_ui.frame:SetWidth(wadj + frm_w)
	end
	
	if _ui.FRAMEANCHOR ~= fr_anchor then
		_ui.frame:ClearPoint(_ui.FRAMEANCHOR)
		_ui.FRAMEANCHOR = fr_anchor
	end
	_ui.frame:SetPoint(fr_anchor, _ui.button, fr_target, 0,0)
end

function DK.ResizeButtons()
	local frmw = 0
	for k, v in pairs(docked_icons) do
		v.l:SetFontSize(BUTTON_SIZE/3)
		frmw = math.max(frmw, v.l:GetWidth())
		v.f:SetHeight(BUTTON_SIZE)
		v.f:SetWidth(BUTTON_SIZE)
		v.o:SetHeight(BUTTON_SIZE)
		v.o:SetWidth(BUTTON_SIZE)
	end
	
	_ui.frmw = frmw
	DK.FixDockedLayout()
end

function DK.Register(name, frame, close)
	print(string.format("Registering %s", name))
	local label = UI.CreateFrame("Text", "label", _ui.frame)
	label:SetFontSize(16)
	label:SetText(name.."  ")
	_ui.frmw = math.max(_ui.frmw, label:GetWidth())
	label:SetPoint("CENTERLEFT", frame, "CENTERRIGHT", 2,0)
	local overlay = UI.CreateFrame("Texture", "overlay", _ui.frame)
	overlay:SetVisible(false)
	overlay:SetTexture(addon.identifier, "images/overlay.png")
	overlay:SetHeight(BUTTON_SIZE)
	overlay:SetWidth(BUTTON_SIZE)
	overlay:SetPoint("TOPLEFT", frame, "TOPLEFT", 0,0)
	overlay:SetBackgroundColor(1,0,0,0.5)
	overlay:SetLayer(frame:GetLayer()+1)
	overlay:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if _ui.cfg_docklockcb:GetChecked() and Docker_Settings.mode == DK_MODE_BAR then
			self.MouseDown = true
			local md = Inspect.Mouse()
			self.sx = md.x - overlay:GetLeft()
			self.sy = md.y - overlay:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Down")

	overlay:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local md = Inspect.Mouse()
			ny = md.y - self.sy
			nx = md.x - self.sx
			overlay:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx, ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")
	
	overlay:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		local n = name
		if self.MouseDown then
			self.MouseDown = false
			local md = Inspect.Mouse()
			local s,p			
			if Docker_Settings.dock_edge == DK_DOCK_LT or Docker_Settings.dock_edge == DK_DOCK_RT then
				sc = UIParent:GetHeight()
				p = md.y
			else
				sc = UIParent:GetWidth()
				p = md.x
			end
			if p/sc <= 1/3 then
				Docker_Settings.dock_preferences[n] = DK_POS_LT
			elseif p/sc >= 2/3 then
				Docker_Settings.dock_preferences[n] = DK_POS_RB
			else
				Docker_Settings.dock_preferences[n] = DK_POS_MM
			end
			DK.FixDockedLayout()
			overlay:SetPoint("TOPLEFT", frame, "TOPLEFT", 0,0)
		end
	end, "Event.UI.Input.Mouse.Left.Up")
		
	table.insert(docked_icons, {n=name, f=frame, l=label, a="CENTER", c=close, o=overlay})
	frame:SetHeight(BUTTON_SIZE)
	frame:SetWidth(BUTTON_SIZE)
	frame:SetParent(_ui.frame)
	DK.FixDockedLayout()
end

function DK.CreateUI()
	_ui.context = UI.CreateContext(addon.identifier)
	
	_ui.button = UI.CreateFrame("Texture", "_ui.button", _ui.context)
	_ui.button:SetPoint("CENTER", UIParent, "CENTER",0,0)
	local x = _ui.button:GetLeft()
	local y = _ui.button:GetTop()
	_ui.button:ClearPoint("CENTERRIGHT")
	_ui.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x,y)
	_ui.button:SetHeight(32)
	_ui.button:SetWidth(32)
	_ui.button:SetLayer(1000)

	_ui.button:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if Docker_Settings.locked or Docker_Settings.mode == DK_MODE_BAR then
			_ui.cfg:SetVisible(true)
		end
	end, "Event.UI.Input.Mouse.Left.Click")
	
	_ui.button:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		if Docker_Settings.mode == DK_MODE_FLOAT and Docker_Settings.locked == false then
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - _ui.button:GetLeft()
			self.sy = mouseData.y - _ui.button:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Down")

	_ui.button:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if Docker_Settings.mode == DK_MODE_FLOAT and Docker_Settings.locked == false and self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			if self.sx and self.sy then
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
			else
				self.sx = mouseData.x - _ui.button:GetLeft()
				self.sy = mouseData.y - _ui.button:GetTop()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
			end
			_ui.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	_ui.button:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		if Docker_Settings.mode == DK_MODE_FLOAT and Docker_Settings.locked == false then
			Docker_Settings.mmx = _ui.button:GetLeft()
			Docker_Settings.mmy = _ui.button:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Up")
	
	_ui.button:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		self.MouseDown = false
		if Docker_Settings.mode == DK_MODE_FLOAT then
			Docker_Settings.locked = not Docker_Settings.locked
			if Docker_Settings.locked then
				_ui.button:SetTexture("Rift", "btn_ReturnMail_(normal).png.dds")
				_ui.frame:SetVisible(true)
			else
				_ui.button:SetTexture("Rift", "btn_ReturnMail_(disable).png.dds")
				_ui.frame:SetVisible(false)
			end
		end
	end, "Event.UI.Input.Mouse.Right.Click")
	
	_ui.button:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
		if Docker_Settings.locked and Docker_Settings.mode == DK_MODE_FLOAT then
			_ui.frame:SetVisible(true)
		end
	end, "Event.UI.Input.Mouse.Cursor.In")

	_ui.button:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Docker_Settings.locked then
			Docker_Settings.button_size = Docker_Settings.button_size+1
			BUTTON_SIZE = Docker_Settings.button_size
			DK.ResizeButtons()
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	_ui.button:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Docker_Settings.locked then
			if Docker_Settings.button_size > 12 then
				Docker_Settings.button_size = Docker_Settings.button_size-1
				BUTTON_SIZE = Docker_Settings.button_size
				DK.ResizeButtons()
			end
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")
	
	_ui.frame = UI.CreateFrame("Frame", "_ui.frame", _ui.context)
	_ui.frame:SetPoint("BOTTOMLEFT", _ui.button, "TOPLEFT", 0,0)
	_ui.FRAMEANCHOR = "BOTTOMLEFT"
	_ui.frame:SetLayer(-1)
	_ui.frame:SetVisible(false)
	
	_ui.menubar = {}
	_ui.menubar.bar = UI.CreateFrame("Frame", "_ui.menubar.bar", _ui.context)
	_ui.menubar.bar:SetVisible(false)
	_ui.menubar.bar:SetWidth(UIParent:GetWidth())
	_ui.menubar.bar:SetHeight(32)
	_ui.menubar.bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0,0)
	_ui.menubar.baranchor = "TOPLEFT"
	_ui.menubar.bar:SetLayer(-100)
	
	_ui.menubar.container_l = UI.CreateFrame("Frame", "_ui.menubar.container_l", _ui.menubar.bar)
	_ui.menubar.container_l:SetWidth(200)
	_ui.menubar.container_l:SetHeight(32)
	_ui.menubar.container_l:SetLayer(-90)
	container_frame_l = _ui.menubar.container_l
	
	_ui.menubar.container_c = UI.CreateFrame("Frame", "_ui.menubar.container_c", _ui.menubar.bar)
	_ui.menubar.container_c:SetWidth(200)
	_ui.menubar.container_c:SetHeight(32)
	_ui.menubar.container_c:SetLayer(-90)
	container_frame_c = _ui.menubar.container_c
	
	_ui.menubar.container_r = UI.CreateFrame("Frame", "_ui.menubar.container_r", _ui.menubar.bar)
	_ui.menubar.container_r:SetWidth(200)
	_ui.menubar.container_r:SetHeight(32)
	_ui.menubar.container_r:SetLayer(-90)
	container_frame_r = _ui.menubar.container_r
	
	_ui.cfg = UI.CreateFrame("RiftWindow", "_ui.cfg", _ui.context)
	_ui.cfg:SetVisible(false)
	_ui.cfg:SetWidth(256)
	_ui.cfg:SetHeight(256)
	_ui.cfg:SetTitle("Docker Configuration")
	_ui.cfg:SetLayer(1)
	
	local cfgborder = _ui.cfg:GetBorder()
	
	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - _ui.cfg:GetLeft()
		self.sy = mouseData.y - _ui.cfg:GetTop()
	end, "Event.UI.Input.Mouse.Left.Down")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			_ui.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		if self.MouseDown then
			self.MouseDown = false
		end
		Docker_Settings.cfgx = _ui.cfg:GetLeft()
		Docker_Settings.cfgy = _ui.cfg:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")	
	
	_ui.cfg_close = UI.CreateFrame("RiftButton", "_ui.cfg_close", _ui.cfg)
	_ui.cfg_close:SetSkin("close")
	_ui.cfg_close:SetLayer(2)
	_ui.cfg_close:SetPoint("TOPRIGHT", _ui.cfg, "TOPRIGHT", -6,6)

	_ui.cfg_close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		_ui.cfg:SetVisible(false)
		_ui.cfg_docklockcb:SetChecked(false)
	end, "Event._ui.Input.Mouse.Left.Click")

	_ui.cfg_modetxt = UI.CreateFrame("Text", "_ui.cfg_modetxt", _ui.cfg)
	_ui.cfg_modetxt:SetFontSize(14)
	_ui.cfg_modetxt:SetText("Mode: ")
	_ui.cfg_modetxt:SetLayer(10)

	local ht = _ui.cfg_modetxt:GetHeight()
	local trl, trt, trr, trb
	trl, trt, trr, trb = _ui.cfg:GetTrimDimensions()
	local wf = _ui.cfg:GetWidth()/2
	
	_ui.cfg_modetxt:SetPoint("TOPLEFT", _ui.cfg, "TOPLEFT", trl, trt)

	_ui.cfg_modegrp = Library.LibSimpleWidgets.RadioButtonGroup("DockModeGroup")
	
	_ui.cfg_modegrp.Event.RadioButtonChange = function(self)
		local btnidx = self:GetSelectedIndex()
		if btnidx == DK_MODE_FLOAT then
			_ui.cfg_FLOAT:SetVisible(true)
			_ui.cfg_DOCK:SetVisible(false)
		else
			_ui.cfg_FLOAT:SetVisible(false)
			_ui.cfg_DOCK:SetVisible(true)
		end
		DK.ChangeLayout()
	end

	_ui.cfg_modefloat = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_modefloat", _ui.cfg)
	_ui.cfg_modefloat:SetText("FLOAT")
	_ui.cfg_modefloat:SetPoint("TOPLEFT", _ui.cfg_modetxt, "TOPLEFT", wf, 0)
	
	_ui.cfg_modedock = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_modedock", _ui.cfg)
	_ui.cfg_modedock:SetText("DOCKED")
	_ui.cfg_modedock:SetPoint("TOPLEFT", _ui.cfg_modefloat, "BOTTOMLEFT", 0, 0)

	_ui.cfg_modegrp:AddRadioButton(_ui.cfg_modefloat)
	_ui.cfg_modegrp:AddRadioButton(_ui.cfg_modedock)

	_ui.cfg_FLOAT = UI.CreateFrame("Frame", "_ui.cfg_FLOAT", _ui.cfg)
	_ui.cfg_FLOAT:SetVisible(false)
	_ui.cfg_FLOAT:SetPoint("TOPCENTER", _ui.cfg_modedock, "BOTTOMLEFT", -trl,0)
	_ui.cfg_FLOAT:SetWidth(256-trl-trr)
	_ui.cfg_FLOAT:SetHeight(128)
	_ui.cfg_FLOAT:SetLayer(5)

	_ui.cfg_floattxt = UI.CreateFrame("Text", "_ui.cfg_floattxt", _ui.cfg_FLOAT)
	_ui.cfg_floattxt:SetFontSize(14)
	_ui.cfg_floattxt:SetText("Direction:")
	_ui.cfg_floattxt:SetLayer(10)
	_ui.cfg_floattxt:SetPoint("TOPLEFT", _ui.cfg_FLOAT, "TOPLEFT", 0,0)

	_ui.cfg_floatgrp = Library.LibSimpleWidgets.RadioButtonGroup("DockOptionsGroup")
	_ui.cfg_floatgrp.Event.RadioButtonChange = function(self)
		local btnidx = self:GetSelectedIndex()
		Docker_Settings.float_dir = btnidx
		DK.FixDockedLayout()
	end

	_ui.cfg_fl_U = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_fl_U", _ui.cfg_FLOAT)
	_ui.cfg_fl_U:SetText("Up")
	_ui.cfg_fl_U:SetPoint("TOPLEFT", _ui.cfg_floattxt, "TOPLEFT", wf, 0)
	
	_ui.cfg_fl_R = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_fl_U", _ui.cfg_FLOAT)
	_ui.cfg_fl_R:SetText("Right")
	_ui.cfg_fl_R:SetPoint("TOPLEFT", _ui.cfg_fl_U, "BOTTOMLEFT", 0, 0)
	
	_ui.cfg_fl_D = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_fl_U", _ui.cfg_FLOAT)
	_ui.cfg_fl_D:SetText("Down")
	_ui.cfg_fl_D:SetPoint("TOPLEFT", _ui.cfg_fl_R, "BOTTOMLEFT", 0, 0)
	
	_ui.cfg_fl_L = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_fl_U", _ui.cfg_FLOAT)
	_ui.cfg_fl_L:SetText("Left")
	_ui.cfg_fl_L:SetPoint("TOPLEFT", _ui.cfg_fl_D, "BOTTOMLEFT", 0, 0)
	
	_ui.cfg_floatgrp:AddRadioButton(_ui.cfg_fl_U)
	_ui.cfg_floatgrp:AddRadioButton(_ui.cfg_fl_R)
	_ui.cfg_floatgrp:AddRadioButton(_ui.cfg_fl_D)
	_ui.cfg_floatgrp:AddRadioButton(_ui.cfg_fl_L)

	ht = _ui.cfg_fl_L:GetBottom() - _ui.cfg_fl_U:GetTop()
	
	_ui.cfg_autohidetxt = UI.CreateFrame("Text", "_ui.cfg_autohidetxt", _ui.cfg_FLOAT)
	_ui.cfg_autohidetxt:SetFontSize(14)
	_ui.cfg_autohidetxt:SetText("Autohide:")
	_ui.cfg_autohidetxt:SetLayer(10)
	_ui.cfg_autohidetxt:SetPoint("TOPLEFT", _ui.cfg_FLOAT, "TOPLEFT", 0,ht)
	
	_ui.cfg_autohidecb = UI.CreateFrame("RiftCheckbox", "_ui.cfg_autohidecb", _ui.cfg_FLOAT)
	_ui.cfg_autohidecb:SetChecked(false)
	_ui.cfg_autohidecb:SetPoint("TOPLEFT", _ui.cfg_FLOAT, "TOPLEFT", wf, ht)
	
	_ui.cfg_autohidecb:EventAttach(Event.UI.Checkbox.Change, function(self, h)
		Docker_Settings.autohide = _ui.cfg_autohidecb:GetChecked()
	end, "Event.UI.Checkbox.Change")
	
	_ui.cfg_labelstxt = UI.CreateFrame("Text", "_ui.cfg_labelstxt", _ui.cfg_FLOAT)
	_ui.cfg_labelstxt:SetFontSize(14)
	_ui.cfg_labelstxt:SetText("Show labels (U/D):")
	_ui.cfg_labelstxt:SetLayer(10)
	_ui.cfg_labelstxt:SetPoint("TOPLEFT", _ui.cfg_autohidetxt, "BOTTOMLEFT", 0,0)
	
	_ui.cfg_labelscb = UI.CreateFrame("RiftCheckbox", "_ui.cfg_labelscb", _ui.cfg_FLOAT)
	_ui.cfg_labelscb:SetChecked(false)
	_ui.cfg_labelscb:SetPoint("TOPLEFT", _ui.cfg_autohidecb, "BOTTOMLEFT", 0,0)
	
	_ui.cfg_labelscb:EventAttach(Event.UI.Checkbox.Change, function(self, h)
		Docker_Settings.labels = _ui.cfg_labelscb:GetChecked()
		DK.FixDockedLayout()
	end, "Event.UI.Checkbox.Change")

	ht = _ui.cfg_labelstxt:GetHeight()
	_ui.cfg_autohidecb:SetHeight(ht)
	_ui.cfg_labelscb:SetHeight(ht)
	
	_ui.cfg_DOCK = UI.CreateFrame("Frame", "_ui.cfg_DOCK", _ui.cfg)
	_ui.cfg_DOCK:SetVisible(false)
	_ui.cfg_DOCK:SetPoint("TOPCENTER", _ui.cfg_modedock, "BOTTOMLEFT", -trl,0)
	_ui.cfg_DOCK:SetWidth(256-trl-trr)
	_ui.cfg_DOCK:SetHeight(128)
	_ui.cfg_DOCK:SetLayer(5)

	_ui.cfg_docktxt = UI.CreateFrame("Text", "_ui.cfg_docktxt", _ui.cfg_DOCK)
	_ui.cfg_docktxt:SetFontSize(14)
	_ui.cfg_docktxt:SetText("Attach to:")
	_ui.cfg_docktxt:SetLayer(10)
	_ui.cfg_docktxt:SetPoint("TOPLEFT", _ui.cfg_DOCK, "TOPLEFT", 0,0)

	_ui.cfg_dockgrp = Library.LibSimpleWidgets.RadioButtonGroup("DockOptionsGroup")
	_ui.cfg_dockgrp.Event.RadioButtonChange = function(self)
		local btnidx = self:GetSelectedIndex()
		Docker_Settings.dock_edge = btnidx
		DK.FixDockedLayout()
	end
	
	_ui.cfg_rb_top = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_rb_top", _ui.cfg_DOCK)
	_ui.cfg_rb_top:SetText("Top")
	_ui.cfg_rb_top:SetPoint("TOPLEFT", _ui.cfg_docktxt, "TOPLEFT", wf, 0)

	_ui.cfg_rb_right = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_rb_right", _ui.cfg_DOCK)
	_ui.cfg_rb_right:SetText("Right")
	_ui.cfg_rb_right:SetPoint("TOPLEFT", _ui.cfg_rb_top, "BOTTOMLEFT", 0, 0)

	_ui.cfg_rb_bottom = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_rb_bottom", _ui.cfg_DOCK)
	_ui.cfg_rb_bottom:SetText("Bottom")
	_ui.cfg_rb_bottom:SetPoint("TOPLEFT", _ui.cfg_rb_right, "BOTTOMLEFT", 0, 0)

	_ui.cfg_rb_left = UI.CreateFrame("SimpleRadioButton", "_ui.cfg_rb_left", _ui.cfg_DOCK)
	_ui.cfg_rb_left:SetText("Left")
	_ui.cfg_rb_left:SetPoint("TOPLEFT", _ui.cfg_rb_bottom, "BOTTOMLEFT", 0, 0)
	
	_ui.cfg_dockgrp:AddRadioButton(_ui.cfg_rb_top)
	_ui.cfg_dockgrp:AddRadioButton(_ui.cfg_rb_right)
	_ui.cfg_dockgrp:AddRadioButton(_ui.cfg_rb_bottom)
	_ui.cfg_dockgrp:AddRadioButton(_ui.cfg_rb_left)
	
	ht = _ui.cfg_rb_left:GetBottom() - _ui.cfg_rb_top:GetTop()
	
	_ui.cfg_docklocktxt = UI.CreateFrame("Text", "_ui.cfg_docklocktxt", _ui.cfg_DOCK)
	_ui.cfg_docklocktxt:SetFontSize(14)
	_ui.cfg_docklocktxt:SetText("Unlock icons:")
	_ui.cfg_docklocktxt:SetLayer(10)
	_ui.cfg_docklocktxt:SetPoint("TOPLEFT", _ui.cfg_DOCK, "TOPLEFT", 0,ht)
	
	_ui.cfg_docklockcb = UI.CreateFrame("RiftCheckbox", "_ui.cfg_docklockcb", _ui.cfg_DOCK)
	_ui.cfg_docklockcb:SetChecked(false)
	_ui.cfg_docklockcb:SetPoint("TOPLEFT", _ui.cfg_DOCK, "TOPLEFT", wf, ht)
	
	_ui.cfg_docklockcb:EventAttach(Event.UI.Checkbox.Change, function(self, h)
		local cb = _ui.cfg_docklockcb:GetChecked()
		for k,v in pairs(docked_icons) do
			v.o:SetVisible(cb)
		end
	end, "Event.UI.Checkbox.Change")
	
end

function DK.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if Docker_Settings == nil then Docker_Settings = {}
			Docker_Settings.mmx = _ui.button:GetLeft()
			Docker_Settings.mmy = _ui.button:GetTop()
		end
		
		MergeTable(Docker_Settings, default_settings)
		
		_ui.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Docker_Settings.cfgx,Docker_Settings.cfgy)
		
		if Docker_Settings.mode == DK_MODE_FLOAT then
			_ui.cfg_modegrp:SetSelectedIndex(1, true)
			_ui.cfg_FLOAT:SetVisible(true)
			_ui.cfg_DOCK:SetVisible(false)			
		else
			_ui.cfg_modegrp:SetSelectedIndex(2, true)
			_ui.cfg_FLOAT:SetVisible(false)
			_ui.cfg_DOCK:SetVisible(true)			
		end
		
		_ui.cfg_dockgrp:SetSelectedIndex(Docker_Settings.dock_edge, true)
		_ui.cfg_floatgrp:SetSelectedIndex(Docker_Settings.float_dir, true)
		_ui.cfg_autohidecb:SetChecked(Docker_Settings.autohide)
		_ui.cfg_labelscb:SetChecked(Docker_Settings.labels)
		
		if Docker_Settings.mode == DK_MODE_FLOAT then
			_ui.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Docker_Settings.mmx, Docker_Settings.mmy)
		else
			_ui.button:SetPoint("TOPLEFT", _ui.menubar.bar, "TOPLEFT", 0,0)
			container_frame_l = _ui.button
			_ui.menubar.bar:SetVisible(true)
		end
		
		if Docker_Settings.locked or Docker_Settings.mode == DK_MODE_BAR then
			_ui.button:SetTexture("Rift", "btn_ReturnMail_(normal).png.dds")
			_ui.frame:SetVisible(true)
		else
			_ui.button:SetTexture("Rift", "btn_ReturnMail_(disable).png.dds")
			_ui.frame:SetVisible(false)
		end
				
		BUTTON_SIZE = Docker_Settings.button_size
		DK.ResizeButtons()
		DK.FixDockedLayout()
		
		LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
	end
end

local firstupdate = 0

function DK.Event_System_Update_Begin(h)
	if Docker_Settings.autohide and Docker_Settings.mode == DK_MODE_FLOAT then
		local md = Inspect.Mouse()
		local l = _ui.frame:GetLeft()
		local r = _ui.frame:GetRight()
		local t = _ui.frame:GetTop()
		local b = _ui.frame:GetBottom()
		if md.x < l or md.x > r or md.y < t or md.y > b then
			_ui.frame:SetVisible(false)
		end
	end
end

function DK.Command_Slash_Register(h,args)
	local r = {}
	local numargs = 0
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
	numargs=#r
	if numargs > 0 then
		if r[1] == "reset" then
			if Docker_Settings.mode == DK_MODE_FLOAT then
				if Docker_Settings.mmx < 0 then
					Docker_Settings.mmx = 0
				elseif (Docker_Settings.mmx+32) > UIParent:GetWidth() then
					Docker_Settings.mmx = UIParent:GetWidth()-32
				end
				if Docker_Settings.mmy < 0 then
					Docker_Settings.mmy = 0
				elseif (Docker_Settings.mmy+32) > UIParent:GetHeight() then
					Docker_Settings.mmy = UIParent:GetHeight()-32
				end
			end
		elseif r[1] == "config" then
			_ui.cfg:SetVisible(true)
		end
	end
end

function DK.Event_Addon_Load_End(h,a)
	if a == "KingMolinator" then
		print("KBM Loaded.. Hijack!")
		KBM.Button.Texture:ClearPoint("CENTER")
		KBM.Button.Texture:ClearPoint("TOPLEFT")
		KBM.Button.Drag:SetVisible(false)
		KBM.Button.Drag.Event.RightDown = nil
		KBM.Button.Drag.Event.RightUp = nil
		KBM.Button.Drag.Event.MouseIn = nil
		KBM.Button.Drag.Event.MouseOut = nil
		MINIMAPDOCKER.Register("KBM", KBM.Button.Texture)
	end
end

DK.CreateUI()

Command.Event.Attach(Command.Slash.Register("docker"), DK.Command_Slash_Register, "Command.Slash.Register")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, DK.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.System.Update.Begin, DK.Event_System_Update_Begin, "Event.System.Update.Begin")
--Command.Event.Attach(Event.Addon.Load.End, DK.Event_Addon_Load_End, "Event.Addon.Load.End")

print(string.format("v%s loaded.", addon.toc.Version))